package com.prowidesoftware.brilliance.mx2mt;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.dic.GroupHeader42;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

public abstract class AbstractMx2MtCreator implements Mx2MtCreator {

    private static final Logger logger = LoggerFactory.getLogger(AbstractMx2MtCreator.class);

    protected Map<String, Object> map = new HashMap<>();

    protected AbstractMx2MtCreator withObj(Object obj) {
        map.put(Mx2MtConstants.MX_OBJECT, obj);
        return this;
    }

    protected AbstractMx2MtCreator withOutPutFilePath(String outPutFilePath) {
        if (!StringUtil.isEmpty(outPutFilePath)) {
             map.put(Mx2MtConstants.OUT_PUT_FILE_PATH, outPutFilePath);
        }
        return this;
    }

    protected AbstractMx2MtCreator withExtraMap(Map<String, Object> extraMap) {
        if (extraMap != null && extraMap.size() > 0) {
            logger.info("External Map...");
            extraMap.forEach((key, value) -> {
                logger.info(key + "-" + value);
                map.put(key, value);
            });
        }
        return this;
    }

    protected void generateCommonField(GroupHeader42 grpHdr) {
        if (grpHdr != null) {
            String createDateStr = DateUtil.format(grpHdr.getCreDtTm(), "YYMMdd");
            String createTimeStr = DateUtil.format(grpHdr.getCreDtTm(), "HHmm");
            map.put("createDate", createDateStr);
            map.put("createTime", createTimeStr);
            String outDateStr = DateUtil.formatNow( "YYMMdd");
            String outTimeStr = DateUtil.formatNow("HHmm");
            map.put("outDate", outDateStr);
            map.put("outTime", outTimeStr);
        }
    }

    public Map<String, Object> getMap() {
        return map;
    }

    public void setMap(Map<String, Object> map) {
        this.map = map;
    }
}
