package com.prowidesoftware.brilliance.mx2mt;

import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.mt900910.Mx2Mt900910Creator;
import com.prowidesoftware.swift.model.mx.AbstractMX;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.List;
import java.util.Map;

public class Mx2MtCreateManager {

    private static final Logger logger = LoggerFactory.getLogger(Mx2MtCreateManager.class);

    private String msgType = "";

    private Object mxObj = null;

    public String getMsgType() {
        return msgType;
    }

    public void setMsgType(String msgType) {
        this.msgType = msgType;
    }

    public Object getMxObj() {
        return mxObj;
    }

    public void setMxObj(Object mxObj) {
        this.mxObj = mxObj;
    }

    public void init(String xmlStr) {
        AbstractMX abstractMX = AbstractMX.parse(xmlStr);
        msgType = (abstractMX.getMxId().getBusinessProcess().name()
                + abstractMX.getMxId().getFunctionality()
                + abstractMX.getMxId().getVariant()
                + abstractMX.getMxId().getVersion())
                .trim()
                .replace(".", "");
        mxObj = abstractMX;
    }

    public String mx2mt(File file, String outPutFilePath, Map<String, Object> extraMap) throws Exception {
        logger.info("invoke mx2mt(file) start...");
        String xmlStr = FileUtils.readFileToString(file);
        init(xmlStr);
        logger.info("Xml Message Type = " + msgType);
        logger.info("Xml Message Obj = " + mxObj);
        return convert(outPutFilePath, extraMap);
    }

    public String mx2mt(File file, String outPutFilePath) throws Exception {
        return mx2mt(file, outPutFilePath, null);
    }

    public String mx2mt(File file) throws Exception {
        return mx2mt(file, null, null);
    }

    public String mx2mt(String xmlStr, String outPutFilePath, Map<String, Object> extraMap) throws Exception {
        logger.info("invoke mx2mt(string) start...");
        init(xmlStr);
        logger.info("Xml Message Type = " + msgType);
        logger.info("Xml Message Obj = " + mxObj);
        return convert(outPutFilePath, extraMap);
    }

    public String mx2mt(String xmlStr, String outPutFilePath) throws Exception {
        return mx2mt(xmlStr, outPutFilePath, null);
    }

    public String mx2mt(String xmlStr) throws Exception {
        return mx2mt(xmlStr, null, null);
    }

    public String convert(String outPutFilePath, Map<String, Object> extraMap) throws SwiftException {
        AbstractMx2MtCreator creator = getMx2MtCreator(msgType);
        creator.withObj(mxObj).withOutPutFilePath(outPutFilePath).withExtraMap(extraMap);
        List<Mx2MtFieldsGenerate> generateList = creator.getFieldsGenerateList();
        for (int i=0; i<generateList.size(); i++) {
            AbstractMx2MtFieldsGenerate fieldsGenerate = (AbstractMx2MtFieldsGenerate) generateList.get(i);
            fieldsGenerate.withMap(creator.getMap());
            fieldsGenerate.fieldsGenerate();
        }
        return creator.generateMtMsg();
    }

    public AbstractMx2MtCreator getMx2MtCreator(String msgType) throws SwiftException {
        /*if ("pacs00800102".equals(msgType)) {
            return new Mx2Mt103Creator();
        } else if ("camt05300102".equals(msgType)) {
            return new Mx2Mt950Creator();
        } else*/ if ("camt05400102".equals(msgType)) {
            return new Mx2Mt900910Creator();
        } else {
            throw new SwiftException("Invalid message type");
        }
    }
}
