package com.prowidesoftware.brilliance.mx2mt.mt900910;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtCreator;
import com.prowidesoftware.brilliance.mx2mt.Mx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.mx2mt.mt900910.impl.*;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.brilliance.util.TemplateUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class Mx2Mt900910Creator extends AbstractMx2MtCreator {

    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt900910Creator.class);

    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {
        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();
        fieldsGenerateList.add(new ReceiverBicFieldGenerate());
        fieldsGenerateList.add(new SenderBicFieldGenerate());
        fieldsGenerateList.add(new Field13DGenerate());
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field21Generate());
        fieldsGenerateList.add(new Field25aGenerate());
        fieldsGenerateList.add(new Field32AGenerate());
        fieldsGenerateList.add(new Field50aGenerate());
        fieldsGenerateList.add(new Field52aGenerate());
        fieldsGenerateList.add(new Field56aGenerate());
        fieldsGenerateList.add(new Field72Generate());
        return fieldsGenerateList;
    }

    @Override
    public String generateMtMsg() {
        MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
        if (mxCamt05400102 != null
                && mxCamt05400102.getBkToCstmrDbtCdtNtfctn() != null
                && mxCamt05400102.getBkToCstmrDbtCdtNtfctn().getGrpHdr() != null) {
            generateCommonField(mxCamt05400102.getBkToCstmrDbtCdtNtfctn().getGrpHdr());
        }
        validate();
        map.forEach((key,value)->{
            logger.info(key + "-" + value);
        });
        String outPutFilePath = (String)map.get(Mx2MtConstants.OUT_PUT_FILE_PATH);
        String msg = TemplateUtil.makeFileByTemplate("swift_mt_900_910.vm", map, outPutFilePath, "utf-8");
        return msg;
    }

    public void validate() {
        String mtType = (String)map.get(Mx2MtConstants.MT_TYPE);
        if (Mx2MtConstants.MT_TYPE_910.equals(mtType)) {
            String field50a = (String)map.get("field50a");
            String field52a = (String)map.get("field52a");
            if (StringUtil.isEmpty(field50a) && StringUtil.isEmpty(field52a)) {
                throw new SwiftException("ERROR", "Either field 50a or field 52a must be present.");
            }
        }
    }
}
