package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * <Ntfctn><Acct><Id><IBAN>
 * <Ntfctn><Acct><Id><<Othr><Id>
 */
public class Field25aGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field25aGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field25aGenerate start.");
        try {
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                CashAccount20 acct = ntfctn.getAcct();
                if (acct != null) {
                    AccountIdentification4Choice id = acct.getId();
                    if (id != null) {
                        if (!StringUtil.isEmpty(id.getIBAN())) {
                            map.put("field25a", id.getIBAN());
                        } else {
                            GenericAccountIdentification1 othr = id.getOthr();
                            if (othr != null) {
                                map.put("field25a", othr.getId());
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field25a error");
        }
    }
}
