package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.NumberUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * <Ntfctn><Ntry><ValDt><Dt>
 * <Ntfctn><Ntry><Amt Ccy="XXX">
 */
public class Field32AGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field32AGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field32AGenerate start.");
        try {
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                List<ReportEntry2> ntryList = ntfctn.getNtry();
                if (ntryList != null && ntryList.size() > 0) {
                    ReportEntry2 ntry = ntryList.get(0);
                    String dateStr = "";
                    if (ntry.getValDt() != null) {
                        dateStr = DateUtil.format(ntry.getValDt().getDt(), "YYMMdd");
                    }
                    String ccy = "";
                    String amtStr = "";
                    ActiveOrHistoricCurrencyAndAmount amt = ntry.getAmt();
                    if (amt != null) {
                        ccy = amt.getCcy();
                        amtStr = NumberUtil.formatAmt(amt.getValue(), ccy);
                    }
                    map.put("field32A", dateStr+ccy+amtStr);
                    //judge 900 or 910
                    if (CreditDebitCode.DBIT.equals(ntry.getCdtDbtInd())) {
                        map.put(Mx2MtConstants.MT_TYPE, Mx2MtConstants.MT_TYPE_900);
                    } else if (CreditDebitCode.CRDT.equals(ntry.getCdtDbtInd())) {
                        map.put(Mx2MtConstants.MT_TYPE, Mx2MtConstants.MT_TYPE_910);
                    }
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field32A error");
        }
    }
}
