package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * 50A
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><Dbtr><Pty><Id><OrgId><AnyBIC>
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><DbtrAcct><Id><IBAN>
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><DbtrAcct><Id><Othr><Id>
 *
 * 50K
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><Dbtr><Pty><Nm>
 * <Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties><Dbtr><Pty><PstlAdr><AdrLine>
 */
public class Field50aGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field50aGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        String mtType = (String)map.get("mtType");
        if (!Mx2MtConstants.MT_TYPE_910.equals(mtType)) {
            return;
        }
        logger.info("Field50aGenerate start.");
        try {
            TransactionParty2 rltdPties = getTransactionParty2();
            if (rltdPties == null) {
                logger.info("<Ntfctn><Ntry><NtryDtls><TxDtls><RltdPties> Object is null.");
                return;
            }
            String field50aOption = "";
            String field50a = "";
            PartyIdentification32 dbtr = rltdPties.getDbtr();
            if (dbtr != null) {
                if (dbtr.getId() != null
                    && dbtr.getId().getOrgId() != null
                    && !StringUtil.isEmpty(dbtr.getId().getOrgId().getBICOrBEI())) {
                    String bicCode = dbtr.getId().getOrgId().getBICOrBEI();
                    field50aOption = "A";
                    String iban = "";
                    String otherId = "";
                    CashAccount16 dbtrAcct = rltdPties.getDbtrAcct();
                    if (dbtrAcct.getId() != null) {
                        iban = dbtrAcct.getId().getIBAN();
                        if (StringUtil.isEmpty(iban)) {
                            GenericAccountIdentification1 othr = dbtrAcct.getId().getOthr();
                            if (othr != null) {
                                otherId = othr.getId();
                            }
                        }
                    }
                    if (!StringUtil.isEmpty(iban)) {
                        field50a = "/"+iban + "\r\n" + bicCode;
                    } else if (!StringUtil.isEmpty(otherId)) {
                        field50a = "//CH"+otherId + "\r\n" + bicCode;
                    } else {
                        field50a = bicCode;
                    }
                } else if (!StringUtil.isEmpty(dbtr.getNm())) {
                    field50aOption = "K";
                    field50a = "/" + dbtr.getNm();
                    PostalAddress6 pstlAdr = dbtr.getPstlAdr();
                    if (pstlAdr != null) {
                        List<String> adrLineList = pstlAdr.getAdrLine();
                        if (adrLineList.size() > 0) {
                            for (int i=0; i<adrLineList.size(); i++) {
                                if (i == 4) {
                                    break;
                                }
                                String adrLine = adrLineList.get(i);
                                if (adrLine.length() > 35) {
                                    adrLine = adrLine.substring(0, 35);
                                }
                                field50a += Mx2MtConstants.NEW_LINE+adrLine;
                            }
                        }
                    }
                }
            }
            if (StringUtil.isNotEmpty(field50aOption) && StringUtil.isNotEmpty(field50a)) {
                map.put("field50aOption", field50aOption);
                map.put("field50a", field50a);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field50a error");
        }
    }

    private TransactionParty2 getTransactionParty2() {
        try {
            TransactionParty2 rltdPties = null;
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return null;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                List<ReportEntry2> ntryList = ntfctn.getNtry();
                if (ntryList != null && ntryList.size() > 0) {
                    ReportEntry2 ntry = ntryList.get(0);
                    List<EntryDetails1> ntryDtlsList = ntry.getNtryDtls();
                    if (ntryDtlsList != null && ntryDtlsList.size() > 0) {
                        EntryDetails1 ntryDtls = ntryDtlsList.get(0);
                        List<EntryTransaction2> txDtlsList = ntryDtls.getTxDtls();
                        if (txDtlsList != null && txDtlsList.size() > 0) {
                            EntryTransaction2 txDtls = txDtlsList.get(0);
                            rltdPties = txDtls.getRltdPties();
                        }
                    }
                }
            }
            return rltdPties;
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field50a error");
        }
    }
}
