package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class Field52aGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field52aGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field52aGenerate start.");
        try {
            BranchAndFinancialInstitutionIdentification4 crDrTrAgt = getCrDrTrAgt();
            if (crDrTrAgt == null) {
                logger.info("<Ntfctn><Ntry><NtryDtls><TxDtls><RltdAgts><DbtrAgt>or<CdtrAgt> Object is null.");
                return;
            }
            String field52aOption = "";
            String field52a = "";
            FinancialInstitutionIdentification7 finInstnId = crDrTrAgt.getFinInstnId();
            if (finInstnId != null) {
                String bicCode = finInstnId.getBIC();
                if (StringUtil.isNotEmpty(bicCode)) {
                    field52aOption = "A";
                    String memberId = "";
                    ClearingSystemMemberIdentification2 clrSysMmbId = finInstnId.getClrSysMmbId();
                    if (clrSysMmbId != null) {
                        memberId = clrSysMmbId.getMmbId();
                    }
                    if (StringUtil.isNotEmpty(memberId)) {
                        field52a = "/" + memberId + Mx2MtConstants.NEW_LINE + bicCode;
                    } else {
                        field52a = bicCode;
                    }
                } else if (!StringUtil.isEmpty(finInstnId.getNm())){
                    field52aOption = "D";
                    field52a = "/" + finInstnId.getNm();
                    PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                    if (pstlAdr != null) {
                        List<String> adrLineList = pstlAdr.getAdrLine();
                        if (adrLineList.size() > 0) {
                            for (int i=0; i<adrLineList.size(); i++) {
                                if (i == 4) {
                                    break;
                                }
                                String adrLine = adrLineList.get(i);
                                if (adrLine.length() > 35) {
                                    adrLine = adrLine.substring(0, 35);
                                }
                                if (i==0 && StringUtil.isEmpty(field52a)) {
                                    field52a += adrLine;
                                } else {
                                    field52a += Mx2MtConstants.NEW_LINE+adrLine;
                                }
                            }
                        }
                    }
                }
            }
            if (StringUtil.isNotEmpty(field52aOption) && StringUtil.isNotEmpty(field52a)) {
                map.put("field52aOption", field52aOption);
                map.put("field52a", field52a);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field52a error");
        }
    }

    private BranchAndFinancialInstitutionIdentification4 getCrDrTrAgt() {
        try {
            BranchAndFinancialInstitutionIdentification4 crDrTrAgt = null;
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return null;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                List<ReportEntry2> ntryList = ntfctn.getNtry();
                if (ntryList != null && ntryList.size() > 0) {
                    ReportEntry2 ntry = ntryList.get(0);
                    List<EntryDetails1> ntryDtlsList = ntry.getNtryDtls();
                    if (ntryDtlsList != null && ntryDtlsList.size() > 0) {
                        EntryDetails1 ntryDtls = ntryDtlsList.get(0);
                        List<EntryTransaction2> txDtlsList = ntryDtls.getTxDtls();
                        if (txDtlsList != null && txDtlsList.size() > 0) {
                            EntryTransaction2 txDtls = txDtlsList.get(0);
                            TransactionAgents2 rltdAgts = txDtls.getRltdAgts();
                            if (rltdAgts != null) {
                                /*String mtType = (String)map.get("mtType");
                                if (Mx2MtConstants.MT_TYPE_900.equals(mtType)) {
                                    crDrTrAgt = rltdAgts.getDbtrAgt();
                                } else if (Mx2MtConstants.MT_TYPE_910.equals(mtType)) {
                                    crDrTrAgt = rltdAgts.getCdtrAgt();
                                }*/
                                crDrTrAgt = rltdAgts.getDbtrAgt();
                            }
                        }
                    }
                }
            }
            return crDrTrAgt;
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field52a error");
        }
    }
}
