package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class Field56aGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field56aGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        String mtType = (String)map.get("mtType");
        if (!Mx2MtConstants.MT_TYPE_910.equals(mtType)) {
            return;
        }
        logger.info("Field56aGenerate start.");
        try {
            BranchAndFinancialInstitutionIdentification4 intrmyAgt1 = getIntrmyAgt1();
            if (intrmyAgt1 == null) {
                logger.info("<Ntfctn><Ntry><NtryDtls><TxDtls><RltdAgts><IntrmyAgt1> Object is null.");
                return;
            }
            String field56aOption = "";
            String field56a = "";
            FinancialInstitutionIdentification7 finInstnId = intrmyAgt1.getFinInstnId();
            if (finInstnId != null) {
                String bicCode = finInstnId.getBIC();
                if (StringUtil.isNotEmpty(bicCode)) {
                    field56aOption = "A";
                    String memberId = "";
                    ClearingSystemMemberIdentification2 clrSysMmbId = finInstnId.getClrSysMmbId();
                    if (clrSysMmbId != null) {
                        memberId = clrSysMmbId.getMmbId();
                    }
                    if (StringUtil.isNotEmpty(memberId)) {
                        field56a = "/" + memberId + Mx2MtConstants.NEW_LINE + bicCode;
                    } else {
                        field56a = bicCode;
                    }
                } else if (!StringUtil.isEmpty(finInstnId.getNm())){
                    field56aOption = "D";
                    field56a = "/" + finInstnId.getNm();
                    PostalAddress6 pstlAdr = finInstnId.getPstlAdr();
                    if (pstlAdr != null) {
                        List<String> adrLineList = pstlAdr.getAdrLine();
                        if (adrLineList.size() > 0) {
                            for (int i=0; i<adrLineList.size(); i++) {
                                if (i == 4) {
                                    break;
                                }
                                String adrLine = adrLineList.get(i);
                                if (adrLine.length() > 35) {
                                    adrLine = adrLine.substring(0, 35);
                                }
                                if (i==0 && StringUtil.isEmpty(field56a)) {
                                    field56a += adrLine;
                                } else {
                                    field56a += Mx2MtConstants.NEW_LINE+adrLine;
                                }
                            }
                        }
                    }
                }
            }
            if (StringUtil.isNotEmpty(field56aOption) && StringUtil.isNotEmpty(field56a)) {
                map.put("field56aOption", field56aOption);
                map.put("field56a", field56a);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field56a error");
        }
    }

    private BranchAndFinancialInstitutionIdentification4 getIntrmyAgt1() {
        try {
            BranchAndFinancialInstitutionIdentification4 intrmyAgt1 = null;
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return null;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                List<ReportEntry2> ntryList = ntfctn.getNtry();
                if (ntryList != null && ntryList.size() > 0) {
                    ReportEntry2 ntry = ntryList.get(0);
                    List<EntryDetails1> ntryDtlsList = ntry.getNtryDtls();
                    if (ntryDtlsList != null && ntryDtlsList.size() > 0) {
                        EntryDetails1 ntryDtls = ntryDtlsList.get(0);
                        List<EntryTransaction2> txDtlsList = ntryDtls.getTxDtls();
                        if (txDtlsList != null && txDtlsList.size() > 0) {
                            EntryTransaction2 txDtls = txDtlsList.get(0);
                            TransactionAgents2 rltdAgts = txDtls.getRltdAgts();
                            if (rltdAgts != null) {
                                intrmyAgt1 = rltdAgts.getIntrmyAgt1();
                            }
                        }
                    }
                }
            }
            return intrmyAgt1;
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field56a error");
        }
    }
}
