package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class Field72Generate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(Field72Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("Field72Generate start.");
        try {
            EntryTransaction2 txDtls = getEntryTransaction2();
            if (txDtls != null) {
                String addtLtxnInf = txDtls.getAddtlTxInf();
                if (StringUtil.isNotEmpty(addtLtxnInf)) {
                    logger.info(addtLtxnInf);
                    String field72 = StringUtil.getStringByEnter(addtLtxnInf, 35, 6);
                    map.put("field72", field72);
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field56a error");
        }
    }

    private EntryTransaction2 getEntryTransaction2() {
        try {
            EntryTransaction2 txDtls = null;
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return null;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                List<ReportEntry2> ntryList = ntfctn.getNtry();
                if (ntryList != null && ntryList.size() > 0) {
                    ReportEntry2 ntry = ntryList.get(0);
                    List<EntryDetails1> ntryDtlsList = ntry.getNtryDtls();
                    if (ntryDtlsList != null && ntryDtlsList.size() > 0) {
                        EntryDetails1 ntryDtls = ntryDtlsList.get(0);
                        List<EntryTransaction2> txDtlsList = ntryDtls.getTxDtls();
                        if (txDtlsList != null && txDtlsList.size() > 0) {
                            return txDtlsList.get(0);
                        }
                    }
                }
            }
            return txDtls;
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 Field72 error");
        }
    }
}
