package com.prowidesoftware.brilliance.mx2mt.mt900910.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05400102;
import com.prowidesoftware.swift.model.mx.dic.AccountNotification2;
import com.prowidesoftware.swift.model.mx.dic.BankToCustomerDebitCreditNotificationV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class SenderBicFieldGenerate extends AbstractMx2MtFieldsGenerate {

    private static final Logger logger = LoggerFactory.getLogger(SenderBicFieldGenerate.class);

    /**
     * <Stmt><Acct><Ownr><Id><OrgId><AnyBIC>  BICOrBEI
     * @throws SwiftException
     */

    @Override
    public void fieldsGenerate() throws SwiftException {
        logger.info("SenderBicFieldGenerate start.");
        try {
            MxCamt05400102 mxCamt05400102 = (MxCamt05400102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05400102 == null || mxCamt05400102.getBkToCstmrDbtCdtNtfctn() == null) {
                return;
            }
            BankToCustomerDebitCreditNotificationV02 bkToCstmrDbtCdtNtfctn = mxCamt05400102.getBkToCstmrDbtCdtNtfctn();
            List<AccountNotification2> ntfctnList = bkToCstmrDbtCdtNtfctn.getNtfctn();
            if (ntfctnList != null && ntfctnList.size() > 0) {
                AccountNotification2 ntfctn = ntfctnList.get(0);
                if (ntfctn != null
                        && ntfctn.getAcct() != null
                        && ntfctn.getAcct().getOwnr() != null
                        && ntfctn.getAcct().getOwnr().getId() != null
                        && ntfctn.getAcct().getOwnr().getId().getOrgId() != null) {
                    String senderBic = ntfctn.getAcct().getOwnr().getId().getOrgId().getBICOrBEI();
                    String mtSenderBic = processBicCode(senderBic);
                    if (!StringUtil.isEmpty(mtSenderBic)) {
                        map.put("senderBic", mtSenderBic);
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT900/910 SenderBic error");
        }
    }

}
