package com.prowidesoftware.brilliance.mx2mt.mt940950;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtCreator;
import com.prowidesoftware.brilliance.mx2mt.Mx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.mx2mt.mt940950.impl.*;
import com.prowidesoftware.brilliance.util.TemplateUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05300102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class Mx2Mt950Creator extends AbstractMx2MtCreator {

    private static final Logger logger = LoggerFactory.getLogger(Mx2Mt950Creator.class);

    /**
     * Object MxCamt05300102
     * receiverBic
     * senderBic
     * field20
     * field25a
     * field28c
     * field60Option
     * field60F
     * field60A
     * field61List
     * field62Option
     * field62F
     * field62M
     * field64
     */
    @Override
    public List<Mx2MtFieldsGenerate> getFieldsGenerateList() {

        List<Mx2MtFieldsGenerate> fieldsGenerateList = new ArrayList<>();

        if (getMtMsg()){
            fieldsGenerateList.add(new Field21Generate());
            fieldsGenerateList.add(new Field65Generate());
            fieldsGenerateList.add(new Field86Generate());
        }
        fieldsGenerateList.add(new ReceiverBicFieldGenerate());
        fieldsGenerateList.add(new SenderBicFieldGenerate());
        fieldsGenerateList.add(new Field20Generate());
        fieldsGenerateList.add(new Field25aGenerate());
        fieldsGenerateList.add(new Field28CGenerate());
        fieldsGenerateList.add(new Field60Generate());
        fieldsGenerateList.add(new Field61ListGenerate());
        fieldsGenerateList.add(new Field62Generate());
        fieldsGenerateList.add(new Field64Generate());

        return fieldsGenerateList;
    }

    @Override
    public String generateMtMsg() {
        MxCamt05300102 mxCamt05300102 =  (MxCamt05300102)map.get(Mx2MtConstants.MX_OBJECT);
        if (mxCamt05300102 != null
                && mxCamt05300102.getBkToCstmrStmt() != null
                && mxCamt05300102.getBkToCstmrStmt().getGrpHdr() != null) {
            generateCommonField(mxCamt05300102.getBkToCstmrStmt().getGrpHdr());
        }
            map.forEach((key, value) -> {
            logger.info(key + "-" + value);
        });
        String outPutFilePath = (String) map.get(Mx2MtConstants.OUT_PUT_FILE_PATH);
        String template;
        if (getMtMsg())
            template = "swift_mt_940.vm";
        else
            template = "swift_mt_950.vm";

        String msg = TemplateUtil.makeFileByTemplate(template, map, outPutFilePath, "utf-8");
        return msg;
    }

    /**
     * 符合以下其中一个条件，则为940，否则为950报文
     * （1）<Stmt><Ntry>[j]数组中，节点<AddtlNtryInf>有值
     * （2）<Stmt><Bal>[i]数组中，节点<Tp><CdOrPrtry><Cd>值为“FWAV”
     * （3）节点<Stmt><AddtlStmtInf>有值
     * @return true:940, false:950
     */
    public boolean getMtMsg() {
        try {
            MxCamt05300102 mxCamt05300102 = (MxCamt05300102) map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05300102 == null || mxCamt05300102.getBkToCstmrStmt() == null) {
                return false;
            }
            BankToCustomerStatementV02 bkToCstmrStmt = mxCamt05300102.getBkToCstmrStmt();
            List<AccountStatement2> stmtList = bkToCstmrStmt.getStmt();
            if (stmtList != null && stmtList.size() > 0) {
                AccountStatement2 stmt = stmtList.get(0);
                List<ReportEntry2> ntryList = stmt.getNtry();
                if (ntryList.size() > 0 && StringUtils.isNotEmpty(ntryList.get(0).getAddtlNtryInf()))
                    return true;

                List<CashBalance3> balList = stmt.getBal();
                if (balList != null && balList.size() > 0) {
                    for (CashBalance3 tmpBal : balList) {
                        if (tmpBal != null
                                && tmpBal.getTp() != null
                                && tmpBal.getTp().getCdOrPrtry() != null
                                && BalanceType12Code.FWAV.equals(tmpBal.getTp().getCdOrPrtry().getCd())) {
                            return true;
                        }
                    }
                }
                if (StringUtils.isNotEmpty(stmt.getAddtlStmtInf()))
                    return true;

                return false;

            }
        } catch (Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Mx2Mt950Creator.getMtMsg() error");
        }
        return false;
    }
}
