package com.prowidesoftware.brilliance.mx2mt.mt940950.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05300102;
import com.prowidesoftware.swift.model.mx.dic.AccountIdentification4Choice;
import com.prowidesoftware.swift.model.mx.dic.AccountStatement2;
import com.prowidesoftware.swift.model.mx.dic.BankToCustomerStatementV02;
import com.prowidesoftware.swift.model.mx.dic.GenericAccountIdentification1;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * <Stmt><Acct><Id><IBAN>
 * <Stmt><Acct><Id><Othr><Id>
 */
public class Field25aGenerate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(Field25aGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        logger.info("Field25aGenerate start.");
        try {
            MxCamt05300102 mxCamt05300102 =  (MxCamt05300102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05300102 == null || mxCamt05300102.getBkToCstmrStmt() == null) {
                return;
            }
            BankToCustomerStatementV02 bkToCstmrStmt = mxCamt05300102.getBkToCstmrStmt();
            List<AccountStatement2> stmtList = bkToCstmrStmt.getStmt();
            if (stmtList != null && stmtList.size() > 0) {
                AccountStatement2 accountStatement2 = stmtList.get(0);
                if (accountStatement2.getAcct() != null
                    && accountStatement2.getAcct().getId() != null) {
                    AccountIdentification4Choice id = accountStatement2.getAcct().getId();
                    if (!StringUtil.isEmpty(id.getIBAN())) {
                        map.put("field25a", id.getIBAN());
                    } else {
                        GenericAccountIdentification1 othr = id.getOthr();
                        if (othr != null) {
                            map.put("field25a", othr.getId());
                        }
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT940/MT950 Field25a error");
        }
    }
}
