package com.prowidesoftware.brilliance.mx2mt.mt940950.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05300102;
import com.prowidesoftware.swift.model.mx.dic.AccountStatement2;
import com.prowidesoftware.swift.model.mx.dic.BankToCustomerStatementV02;
import com.prowidesoftware.swift.model.mx.dic.GroupHeader42;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.List;

/**
 * <GrpHdr><MsgPgntn><PgNb>
 * <Stmt><ElctrncSeqNb>
 */
public class Field28CGenerate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(Field28CGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        logger.info("Field28CGenerate start.");
        try {
            MxCamt05300102 mxCamt05300102 =  (MxCamt05300102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05300102 == null || mxCamt05300102.getBkToCstmrStmt() == null) {
                return;
            }
            BankToCustomerStatementV02 bkToCstmrStmt = mxCamt05300102.getBkToCstmrStmt();
            BigDecimal seqNb = null;
            List<AccountStatement2> stmtList = bkToCstmrStmt.getStmt();
            if (stmtList != null && stmtList.size() > 0) {
                AccountStatement2 accountStatement2 = stmtList.get(0);
                if (accountStatement2 != null) {
                    seqNb = accountStatement2.getElctrncSeqNb();
                }
            }
            String pgNb = "";
            GroupHeader42 grpHdr = bkToCstmrStmt.getGrpHdr();
            if (grpHdr != null && grpHdr.getMsgPgntn() != null) {
                pgNb = grpHdr.getMsgPgntn().getPgNb();
            }
            if (seqNb == null && StringUtil.isEmpty(pgNb)) {
                throw new SwiftException("ERROR", "Convert MT950 field28C is required!");
            } else {
                if (seqNb == null) { //only <Stmt><ElctrncSeqNb>
                    map.put("field28C", pgNb);
                } else if (StringUtil.isEmpty(pgNb)) { //only <GrpHdr><MsgPgntn><PgNb>
                    map.put("field28C", seqNb.toString());
                } else { //<Stmt><ElctrncSeqNb> + "/" + GrpHdr><MsgPgntn><PgNb>
                    map.put("field28C", seqNb.toString() + "/" + pgNb);
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT940/MT950 field28C error");
        }
    }
}
