package com.prowidesoftware.brilliance.mx2mt.mt940950.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.NumberUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05300102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.math.BigDecimal;
import java.util.List;

/**
 * <Stmt><Bal>[i]<Tp><CdOrPrtry><Cd>
 * <Stmt><Bal>[i]<CdtDbtInd>
 * <Stmt><Bal>[i]<Dt><Dt>
 * <Stmt><Bal>[i]<Amt Ccy=“XXX”>
 * i = 1
 */
public class Field60Generate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(Field60Generate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        logger.info("Field60Generate start.");
        try {
            MxCamt05300102 mxCamt05300102 =  (MxCamt05300102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05300102 == null || mxCamt05300102.getBkToCstmrStmt() == null) {
                return;
            }
            BankToCustomerStatementV02 bkToCstmrStmt = mxCamt05300102.getBkToCstmrStmt();
            List<AccountStatement2> stmtList = bkToCstmrStmt.getStmt();
            if (stmtList != null && stmtList.size() > 0) {
                AccountStatement2 stmt = stmtList.get(0);
                List<CashBalance3> balList = stmt.getBal();
                if (balList != null && balList.size() > 0) {
                    CashBalance3 bal = balList.get(0);
                    String filed60Option = "";
                    if (bal != null && bal.getTp() != null
                        && bal.getTp().getCdOrPrtry() != null) {
                        if (BalanceType12Code.PRCD.equals(bal.getTp().getCdOrPrtry().getCd())) {
                            filed60Option = "F";
                        } else if (BalanceType12Code.ITBD.equals(bal.getTp().getCdOrPrtry().getCd())) {
                            filed60Option = "M";
                        } else {
                            throw new SwiftException("ERROR", "cannot find filed60Option.");
                        }
                        map.put("filed60Option", filed60Option);
                    }
                    if (bal != null) {
                        String crdr = "";
                        if (CreditDebitCode.CRDT.equals(bal.getCdtDbtInd())) {
                            crdr = Mx2MtConstants.DRCRMARK_C;
                        } else if (CreditDebitCode.DBIT.equals(bal.getCdtDbtInd())) {
                            crdr = Mx2MtConstants.DRCRMARK_D;
                        } else {
                            throw new SwiftException("ERROR", "cannot find filed60 D/C Mark.");
                        }
                        String dateStr = "";
                        if (bal.getDt() != null) {
                            dateStr = DateUtil.format(bal.getDt().getDt(), "YYMMdd");
                        }
                        String ccy = "";
                        String amtStr = "";
                        ActiveOrHistoricCurrencyAndAmount ccyAndAmt = bal.getAmt();
                        if (ccyAndAmt != null) {
                            ccy = ccyAndAmt.getCcy();
                            BigDecimal amt = ccyAndAmt.getValue();
                            amtStr = NumberUtil.formatAmt(amt, ccy);
                        }
                        map.put("filed60", crdr+dateStr+ccy+amtStr);
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT940/MT950 Field60Generate error");
        }
    }
}
