package com.prowidesoftware.brilliance.mx2mt.mt940950.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.DateUtil;
import com.prowidesoftware.brilliance.util.NumberUtil;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05300102;
import com.prowidesoftware.swift.model.mx.dic.*;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * <Stmt><Ntry>[j]<ValDt><Dt>
 * <Stmt><Ntry>[j]<BookgDt><Dt>
 * <Stmt><Ntry>[j]<CdtDbtInd>
 * <Stmt><Ntry>[j]<RvslInd>
 * <Stmt><Ntry>[j]<Sts>
 * <Stmt><Ntry>[j]<Amt Ccy=“XXX”>
 * <Stmt><Ntry>[j]<BkTxCd><Prtry><Cd>
 * <Stmt><Ntry>[j]<NtryDtls><TxDtls><Refs><EndToEndId>
 * <Stmt><Ntry>[j]<AcctSvcrRef>
 * <Stmt><Ntry>[j]<NtryDtls><TxDtls><AddtlTxInf>
 */
public class Field61ListGenerate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(Field61ListGenerate.class);

    @Override
    public void fieldsGenerate() throws SwiftException{
        logger.info("Field61ListGenerate start.");
        try {
            MxCamt05300102 mxCamt05300102 =  (MxCamt05300102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05300102 == null || mxCamt05300102.getBkToCstmrStmt() == null) {
                return;
            }
            BankToCustomerStatementV02 bkToCstmrStmt = mxCamt05300102.getBkToCstmrStmt();
            List<AccountStatement2> stmtList = bkToCstmrStmt.getStmt();
            if (stmtList != null && stmtList.size() > 0) {
                AccountStatement2 stmt = stmtList.get(0);
                List<ReportEntry2> ntryList = stmt.getNtry();
                if (ntryList.size() > 0) {
                    List<String> filed61List = new ArrayList<>();
                    for (int i=0; i<ntryList.size(); i++) {
                        ReportEntry2 ntry = ntryList.get(i);
                        String valueDateStr = DateUtil.format(ntry.getValDt().getDt(), "YYMMdd");
                        String entryDateStr = DateUtil.format(ntry.getBookgDt().getDt(), "YYMMdd");
                        String drCrMark = "";
                        if (CreditDebitCode.CRDT.equals(ntry.getCdtDbtInd())) {
                            if (Boolean.TRUE.equals(ntry.isRvslInd())) {
                                drCrMark = Mx2MtConstants.DRCRMARK_RD;
                            } else {
                                drCrMark = Mx2MtConstants.DRCRMARK_C;
                            }
                        } else if (CreditDebitCode.DBIT.equals(ntry.getCdtDbtInd())) {
                            if (Boolean.TRUE.equals(ntry.isRvslInd())) {
                                drCrMark = Mx2MtConstants.DRCRMARK_RC;
                            } else {
                                drCrMark = Mx2MtConstants.DRCRMARK_D;
                            }
                        } else {
                            throw new SwiftException("ERROR", "cannot find filed61 D/C Mark.");
                        }
                        String ccy = "";
                        String amtStr = "";
                        ActiveOrHistoricCurrencyAndAmount ccyAndAmt = ntry.getAmt();
                        if (ccyAndAmt != null) {
                            ccy = ccyAndAmt.getCcy();
                            amtStr = NumberUtil.formatAmt(ccyAndAmt.getValue(), ccy);
                        }
                        //TODO
                        String txnTypeAndIdentification = "";
                        BankTransactionCodeStructure4 bkTxCd = ntry.getBkTxCd();
                        if (bkTxCd != null) {
                            ProprietaryBankTransactionCodeStructure1 prtry = bkTxCd.getPrtry();
                            if (prtry != null) {
                                txnTypeAndIdentification = prtry.getCd();
                            }
                        }
                        String referenceForTheAccountOwner = "";
                        String supplementDetails = "";
                        List<EntryDetails1> ntryDtlsList = ntry.getNtryDtls();
                        if (ntryDtlsList.size() > 0) {
                            EntryDetails1 ntryDtls = ntryDtlsList.get(0);
                            List<EntryTransaction2> txDtlsList = ntryDtls.getTxDtls();
                            if (txDtlsList.size() > 0) {
                                EntryTransaction2 txDtls = txDtlsList.get(0);
                                if (txDtls.getRefs() != null) {
                                    referenceForTheAccountOwner = txDtls.getRefs().getEndToEndId();
                                }
                                supplementDetails = txDtls.getAddtlTxInf();
                            }
                        }
                        String referenceForTheAccountServInst = ntry.getAcctSvcrRef();

                        StringBuilder sb = new StringBuilder();
                        sb.append(valueDateStr)
                          .append(entryDateStr)
                          .append(drCrMark)
                          .append(ccy)
                          .append(amtStr)
                          .append(txnTypeAndIdentification)
                          .append(referenceForTheAccountOwner);
                        if (!StringUtil.isEmpty(referenceForTheAccountServInst)) {
                            sb.append("//"+referenceForTheAccountServInst);
                        }
                        if (!StringUtil.isEmpty(supplementDetails)) {
                            sb.append(Mx2MtConstants.NEW_LINE+supplementDetails);
                        }

                        if (StringUtils.isNotEmpty(ntry.getAddtlNtryInf()))
                            sb.append(Mx2MtConstants.NEW_LINE).append(":86:").append(StringUtil.getStringByEnter(ntry.getAddtlNtryInf(), 35, 6));
                        filed61List.add(sb.toString());
                    }
                    if (filed61List.size() > 0) {
                        filed61List.forEach(field61 -> logger.info(":61:"+field61));
                        map.put("filed61List", filed61List);
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT940/MT950 Field61List error");
        }
    }
}
