package com.prowidesoftware.brilliance.mx2mt.mt940950.impl;

import com.prowidesoftware.brilliance.constants.Mx2MtConstants;
import com.prowidesoftware.brilliance.exception.SwiftException;
import com.prowidesoftware.brilliance.mx2mt.AbstractMx2MtFieldsGenerate;
import com.prowidesoftware.brilliance.util.StringUtil;
import com.prowidesoftware.swift.model.mx.MxCamt05300102;
import com.prowidesoftware.swift.model.mx.dic.AccountStatement2;
import com.prowidesoftware.swift.model.mx.dic.BankToCustomerStatementV02;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class ReceiverBicFieldGenerate extends AbstractMx2MtFieldsGenerate{

    private static final Logger logger = LoggerFactory.getLogger(ReceiverBicFieldGenerate.class);

    /**
     * <Stmt><Acct><Svcr><FinInstnId><BICFI>
     * @throws SwiftException
     */
    @Override
    public void fieldsGenerate() throws SwiftException{
        logger.info("ReceiverBicFieldGenerate start.");
        try {
            MxCamt05300102 mxCamt05300102 = (MxCamt05300102)map.get(Mx2MtConstants.MX_OBJECT);
            if (mxCamt05300102 == null || mxCamt05300102.getBkToCstmrStmt() == null) {
                return;
            }
            BankToCustomerStatementV02 bkToCstmrStmt = mxCamt05300102.getBkToCstmrStmt();
            List<AccountStatement2> stmtList = bkToCstmrStmt.getStmt();
            if (stmtList != null && stmtList.size() > 0) {
                AccountStatement2 accountStatement2 = stmtList.get(0);
                if (accountStatement2 != null
                    && accountStatement2.getAcct() != null
                    && accountStatement2.getAcct().getSvcr() != null
                    && accountStatement2.getAcct().getSvcr().getFinInstnId() != null) {
                    String receiverBic = accountStatement2.getAcct().getSvcr().getFinInstnId().getBIC();
                    String mtReceiverBic = processBicCode(receiverBic);
                    if (!StringUtil.isEmpty(mtReceiverBic)) {
                        map.put("receiverBic", mtReceiverBic);
                    }
                }
            }
        } catch(Exception e) {
            logger.error(e.getMessage());
            throw new SwiftException("ERROR", "Convert MT950 ReceiverBic error.");
        }
    }

}
